'''
  Copyright (C) 2013 by Christopher Cooper, Lorena Barba

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
'''
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import os
import numpy
import sys
import math
sys.path.append('../util')
from an_solution import *

def scanOutput(filename):
    
    flag = 0
    for line in file(filename):
        line = line.split()
        if len(line)>0:
            if line[0]=='Converged':
                iterations = int(line[2])
            if line[0]=='Total' and line[1]=='elements':
                N = int(line[-1])
            if line[0]=='Totals:':
                flag = 1
            if line[0]=='Esolv' and flag==1:
                Esolv = float(line[2])
            if line[0]=='Esurf' and flag==1:
                Esurf = float(line[2])
            if line[0]=='Ecoul' and flag==1:
                Ecoul = float(line[2])
            if line[0]=='Time' and flag==1:
                Time = float(line[2])

    return N, iterations, Esolv, Esurf, Ecoul, Time
            

mesh = numpy.array([1,2,4,8])

comm = './main.py PyGBeOrientation_1PGBconvergence/inputs/1PGB_fine.param PyGBeOrientation_1PGBconvergence/inputs/1PGBmut_sensor_d0'
out = 'output_aux'

print 'Runs for molecule + set phi/dphi surface'
Nel = numpy.zeros(len(mesh))
iterations = numpy.zeros(len(mesh))
Esolv = numpy.zeros(len(mesh))
Esurf = numpy.zeros(len(mesh))
Ecoul = numpy.zeros(len(mesh))
Time = numpy.zeros(len(mesh))
for i in range(len(mesh)):
    print 'Start run for mesh '+str(mesh[i])
    cmd = comm + str(mesh[i]) + '_010-200.config > ' + out
    os.system(cmd)
    print 'Scan output file'
    Nel[i],iterations[i],Esolv[i],Esurf[i],Ecoul[i],Time[i] = scanOutput(out)
    print 'Esolv: %f, Esurf: %f'%(Esolv[i], Esurf[i])

total_time = Time

print 'Esolv: ' + str(Esolv)
print 'Esurf: ' + str(Esurf)

Esolv_extra = (Esolv[-3]*Esolv[-1]-Esolv[-2]*Esolv[-2])/(Esolv[-1]-2*Esolv[-2]+Esolv[-3])
Esurf_extra = (Esurf[-1]*Esurf[-3]-Esurf[-2]*Esurf[-2])/(Esurf[-1]-2*Esurf[-2]+Esurf[-3])

Esolv_error = abs(Esolv-Esolv_extra)/abs(Esolv_extra)
Esurf_error = abs(Esurf-Esurf_extra)/abs(Esurf_extra)

print '\nNumber of elements : '+str(Nel)
print 'Number of iteration: '+str(iterations)
print 'Esolv extrapolated : %f kcal/mol'%Esolv_extra
print 'Esolv error        : '+str(Esolv_error)
print 'Esurf extrapolated : %f kcal/mol'%Esurf_extra
print 'Esurf error        : '+str(Esurf_error)
print 'Total time         : '+str(total_time)

ylim2  = [min(Esurf_error)/1.2, max(Esurf_error)*1.2]
ylim = [min(Esolv_error)/1.2, max(Esolv_error)*1.2]

dylim = array([ylim[1]/ylim[0], ylim2[1]/ylim2[0]])

dy = max(dylim)

ylim[1] = ylim[0]*dy
ylim2[1] = ylim2[0]*dy

gs = gridspec.GridSpec(2, 1)

font = {'family':'serif','size':10}
plt.rc('font',**font)
fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(gs[0])
ax2 = fig.add_subplot(gs[1])

N = mesh

asymp_solv = N[-1]*Esolv_error[-1]/N
asymp_surf = N[-1]*Esurf_error[-1]/N

ax.loglog(N, Esolv_error, c='k', marker='o',ls=' ', mfc='w', ms=5, label=r'F$_{\rm solv}$')
ax2.loglog(N, Esurf_error, c='k', marker='^',ls=' ', mfc='w', ms=5, label=r'F$_{\rm surf}$')
ax.loglog(N, asymp_solv, c='k', marker='None', ls=':', lw=0.8, label='')
ax2.loglog(N, asymp_surf, c='k', marker='None', ls=':', lw=0.8, label='')

ax.set_ylim(ylim)
ax2.set_ylim(ylim2)

ax.set_xlim(min(N)/1.2)
ax2.set_xlim(min(N)/1.2)

for axis in ['top','bottom','left','right']:
    ax.spines[axis].set_linewidth(0.5)
    ax2.spines[axis].set_linewidth(0.5)

ax.spines['bottom'].set_visible(False)
ax2.spines['top'].set_visible(False)
ax.xaxis.tick_top()
ax.tick_params(labeltop='off')
ax2.xaxis.tick_bottom()

ax2.legend(loc='upper right', fontsize=10, numpoints=1, handlelength=0.1).get_frame().set_lw(0.3)
ax.legend(loc='upper right', fontsize=10, numpoints=1, handlelength=0.1).get_frame().set_lw(0.3)

loc = (3*N[0]+N[1])/4
tex_loc_solv = array((loc,N[0]*Esolv_error[0]/loc))
tex_loc_surf = array((loc,N[0]*Esurf_error[0]/loc))
tex_angle_solv = math.atan2(log(abs(asymp_solv[-1]-asymp_solv[0])),log(abs(N[-1]-N[0])))*180/math.pi
tex_angle_surf = math.atan2(log(abs(asymp_surf[-1]-asymp_surf[0])),log(abs(N[-1]-N[0])))*180/math.pi
ax.text(tex_loc_solv[0], tex_loc_solv[1]/1.7,r'N$^{-1}$',fontsize=8,rotation=-15,rotation_mode='anchor')
ax2.text(tex_loc_surf[0]*1.2, tex_loc_surf[1]*1.4,r'N$^{-1}$',fontsize=8,rotation=-15,rotation_mode='anchor')
ax2.set_ylabel('Relative error', fontsize=10)
ax2.set_xlabel('Mesh density', fontsize=10)
ax2.yaxis.set_label_coords(0.05, 0.5, transform=fig.transFigure)

d = .015 # how big to make the diagonal lines in axes coordinates
# arguments to pass plot, just so we don't keep repeating them
kwargs = dict(transform=ax.transAxes, color='k', clip_on=False, lw=0.5)
ax.plot((-d,+d),(-d,+d), **kwargs)      # top-left diagonal
ax.plot((1-d,1+d),(-d,+d), **kwargs)    # top-right diagonal

kwargs.update(transform=ax2.transAxes)  # switch to the bottom axes
ax2.plot((-d,+d),(1-d,1+d), **kwargs)   # bottom-left diagonal
ax2.plot((1-d,1+d),(1-d,1+d), **kwargs) # bottom-right diagonal

fig.subplots_adjust(left=0.185, bottom=0.22, right=0.965, top=0.95)
fig.savefig('PyGBeOrientation_1PGBconvergence/1PGBconvergence.pdf',dpi=80,format='pdf')

fig.clf()

xmin = N[0]/1.5
xmax = N[-1]*1.5

lims = array([N[0],N[-1]])
fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)
ax.loglog(N, total_time, c='k', marker='o', ls='', ms=4, mfc='w')
ax.loglog(lims, 750*lims*log(lims), c='k', marker='', ls='--', lw=0.5)
ax.set_xlabel('Mesh density',fontsize=10)
ax.set_ylabel('Total time [s]', fontsize=10)
ax.set_xlim([xmin,xmax])

fig.subplots_adjust(left=0.185, bottom=0.21, right=0.965, top=0.95)
fig.savefig('PyGBeOrientation_1PGBconvergence/time.pdf',dpi=80,format='pdf')
fig.clf()

fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)
ax.semilogx(N, iterations, c='k', marker='o', ls='', ms=4, mfc='w')
ax.set_xlabel('Mesh density',fontsize=10)
ax.set_ylabel('Iterations [s]', fontsize=10)

fig.subplots_adjust(left=0.185, bottom=0.21, right=0.965, top=0.95)
fig.savefig('PyGBeOrientation_1PGBconvergence/iterations.pdf',dpi=80,format='pdf')
fig.clf()


